@@ -19,7 +19,7 @@ cd ..
 : ${UPROGRAM:=S-nail}
 : ${MANUAL:=code-nail.html}
 
-: ${UPLOAD:=steffen@sdaoden.eu:/var/www/localhost/downloads}
+: ${UPLOAD:=steffen@vpn.sdaoden.eu:/var/www/localhost/downloads}
 
 : ${MAILX:='s-nail -Snofollowup-to -Sreply-to=mailx -Ssmime-sign'}
 : ${ACCOUNT:=ich}
@@ -128,12 +128,6 @@ update_release_hook() {
    if [ -z "${grappa}" ] && command -v ${roff} >/dev/null 2>&1; then
       echo 'nail.1: creating HTML manual'
       < nail.1 MDOCMX_ENABLE=1 ${roff} -Thtml -mdoc > /tmp/nail-manual.html
-      echo 'nail.1: creating ASCII cat1 in '"${TMPDIR}"
-      < nail.1 MDOCMX_ENABLE= ${roff} -Tascii -mdoc > "${TMPDIR}"/s-nail.cat1
-      echo 'nail.1: creating mdocmx ASCII xcat1 in '"${TMPDIR}"
-      < nail.1 MDOCMX_ENABLE=1 GROFF_NO_SGR=1 \
-            ${roff} -Tascii -dmx-toc-force=tree -dmx-debug=1 -mdoc |
-         col -b > "${TMPDIR}"/s-nail.xcat1
    fi
    ${git} add nail.1
 
